// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// 5-Way Tactile Switch example, Buttons LOW active
// Connecting to GPIO 34 - 39 of the esp32 requires a pullup resistor(10k) for each switch
// --------------------------------------------------------------------------------------
// Libraries:
// TFT_eSPI --> https://github.com/Bodmer/TFT_eSPI
// --------------------------------------------------------------------------------------
// Documentations:
// Overview ESP32 mcu's   - https://socialcompare.com/en/comparison/esp8266-vs-esp32-vs-esp32-s2
// Espressif ESP32 docu's - https://www.espressif.com/en/support/documents/technical-documents
// --------------------------------------------------------------------------------------
// Device WT32-SC01       - The esp32 device WT32-SC01 can not read pixel from the TFT using HSPI
// Device WT32-SC01       - http://myosuploads3.banggood.com/products/20220111/20220111020446WT32-SC01DataSheetV3.3.pdf
// TFT Device ST7796      - https://cdn.hackaday.io/files/1625926956336128/ST7796S.pdf
// Touch Device FT6336U   - https://focuslcds.com/content/FT6236.pdf
// --------------------------------------------------------------------------------------
// Device TTGO-T-Display  - The esp32 device LILYGO-TTGO-T-Display can read pixel from the TFT
// Device TTGO-T-Display  - https://github.com/Xinyuan-LilyGO/TTGO-T-Display
// TFT Device ST7789V     - https://newhavendisplay.com/content/datasheets/ST7789V.pdf
// **************************************************************************************
#include <TFT_eSPI.h>                           // TFT_espi library
TFT_eSPI tft                    = TFT_eSPI();
TFT_eSprite SpriteTriangleGray  = TFT_eSprite(&tft);
TFT_eSprite SpriteTriangleGreen = TFT_eSprite(&tft);
TFT_eSprite SpriteTriangleRed   = TFT_eSprite(&tft);
TFT_eSprite SpriteCircleGray    = TFT_eSprite(&tft);
TFT_eSprite SpriteCircleGreen   = TFT_eSprite(&tft);

// ######################################################################################
// Change the values to match your system and desires
// ######################################################################################
#define digitalCenter_PIN 04                    // button center, FREE esp32 pin
#define digitalUp_PIN     34                    // button up, FREE esp32 pin
#define digitalDown_PIN   35                    // button down, FREE esp32 pin
#define digitalLeft_PIN   36                    // button left, FREE esp32 pin
#define digitalRight_PIN  39                    // button right, FREE esp32 pin
#define speed 100                               // the update time for the main loop

// Vars
uint8_t       textSize;                         // text size
uint8_t       s;                                // switch pushed, if any
uint8_t       sOld = 0;                         // switch pushed old
int16_t       xSprite;                          // sprite width
int16_t       ySprite;                          // sprite height
int32_t       tftWidth;                         // tft width
int32_t       tftHeight;                        // tft height
uint32_t      update;                           // millis

// **************************************************************************************
void SpriteDummy()
{
  //return;
}

// **************************************************************************************
void SpriteResetTriangleRed360()
{
  tft.setPivot(tftWidth / 2, tftHeight / 2 + tftHeight / 100.0 * 25);
  SpriteTriangleRed.pushRotated(360, 0x0000);
}

// **************************************************************************************
void SpriteResetTriangleRed90() 
{
  tft.setPivot(tftWidth / 2 - tftHeight / 100.0 * 25, tftHeight / 2);
  SpriteTriangleRed.pushRotated(90, 0x0000);
}

// **************************************************************************************
void SpriteResetTriangleRed180()
{
  tft.setPivot(tftWidth / 2, tftHeight / 2 - tftHeight / 100.0 * 25);
  SpriteTriangleRed.pushRotated(180, 0x0000);
}

// **************************************************************************************
void SpriteResetTriangleRed270()
{
  tft.setPivot(tftWidth / 2 + tftHeight / 100.0 * 25, tftHeight / 2);
  SpriteTriangleRed.pushRotated(270, 0x0000);
}

// **************************************************************************************
void SpriteResetCircleGray360()
{
  tft.setPivot(tftWidth / 2, tftHeight / 2);
  SpriteCircleGray.pushRotated(360, 0x0000);
}

// **************************************************************************************
// Sprite reset functions table
void (*SpriteResetFunctions[]) (void) =
{
  SpriteDummy,
  SpriteResetTriangleRed360,
  SpriteResetTriangleRed90,
  SpriteResetTriangleRed180,
  SpriteResetTriangleRed270,
  SpriteResetCircleGray360
};



void setup()
{
  tft.init();                                   // TFT_espi - init library

  digitalWrite(TFT_BL, 0);                      // switch backlight off

  pinMode(digitalCenter_PIN, INPUT);            // GPIO 04, input/output possible
  pinMode(digitalUp_PIN, INPUT);                // GPIO 34, input only
  pinMode(digitalDown_PIN, INPUT);              // GPIO 35, input only
  pinMode(digitalLeft_PIN, INPUT);              // GPIO 36, input only
  pinMode(digitalRight_PIN, INPUT);             // GPIO 39, input only

  systemInit();                                 // init system
  screenInit();                                 // init screen

  SpriteTriangleGrayCreate();                   // create and push sprites
  SpriteTriangleGreenCreate();
  SpriteTriangleRedCreate();
  SpriteCircleGrayCreate();
  SpriteCircleGreenCreate();

  tft.setPivot(tftWidth / 2, tftHeight / 2 + tftHeight / 100.0 * 25);
  SpriteTriangleRed.pushRotated(360, 0x0000);
  tft.setPivot(tftWidth / 2 - tftHeight / 100.0 * 25, tftHeight / 2);
  SpriteTriangleRed.pushRotated(90, 0x0000);
  tft.setPivot(tftWidth / 2, tftHeight / 2 - tftHeight / 100.0 * 25);
  SpriteTriangleRed.pushRotated(180, 0x0000);
  tft.setPivot(tftWidth / 2 + tftHeight / 100.0 * 25, tftHeight / 2);
  SpriteTriangleRed.pushRotated(270, 0x0000);
  tft.setPivot(tftWidth / 2, tftHeight / 2);
  SpriteCircleGray.pushRotated(360, 0x0000);

  digitalWrite(TFT_BL, 1);                      // switch backlight on
}



void loop()
{
  if (update <= millis())
  {
    update = millis() + speed;                  // update time in milliseconds

    s = 0;
    if (digitalRead(digitalDown_PIN) == LOW ){s = 1;}
    if (digitalRead(digitalLeft_PIN) == LOW ){s = 2;}
    if (digitalRead(digitalUp_PIN) == LOW ){s = 3;}
    if (digitalRead(digitalRight_PIN) == LOW ){s = 4;}
    if (digitalRead(digitalCenter_PIN) == LOW){s = 5;}

    switch (s)
    {
      case 0:
      if (sOld != 0)
      {
        SpriteResetFunctions[sOld]();
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "X");
        sOld = 0;
      }
      break;

      case 1:
      if (sOld != 1)
      {
        SpriteResetFunctions[sOld]();
        tft.setPivot(tftWidth / 2, tftHeight / 2 + tftHeight / 100.0 * 25); 
        SpriteTriangleGreen.pushRotated(360, 0x0000);
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "D");
        sOld = 1;
      }
      break;

      case 2:
      if (sOld != 2)
      {
        SpriteResetFunctions[sOld]();
        tft.setPivot(tftWidth / 2 - tftHeight / 100.0 * 25, tftHeight / 2); 
        SpriteTriangleGreen.pushRotated(90, 0x0000);
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "L");
        sOld = 2;
      }
      break;

      case 3:
      if (sOld != 3)
      {
        SpriteResetFunctions[sOld]();
        tft.setPivot(tftWidth / 2, tftHeight / 2 - tftHeight / 100.0 * 25);
        SpriteTriangleGreen.pushRotated(180, 0x0000);
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "U");
        sOld = 3;
      }
      break;

      case 4:
      if (sOld != 4)
      {
        SpriteResetFunctions[sOld]();
        tft.setPivot(tftWidth / 2 + tftHeight / 100.0 * 25, tftHeight / 2);
        SpriteTriangleGreen.pushRotated(270, 0x0000);
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "R");
        sOld = 4;
      }
      break;

      case 5:
      if (sOld != 5)
      {
        SpriteResetFunctions[sOld]();
        tft.setPivot(tftWidth / 2, tftHeight / 2);
        SpriteCircleGreen.pushRotated(360, 0x0000);
        tft.setCursor(tftWidth / 100.0 * 86, tftHeight / 4.0);
        tft.printf("%s", "C");
        sOld = 5;
      }
      break;
    }
  }
}



// **************************************************************************************
// System - init
void systemInit()
{
  tft.setRotation(3);                           // set desired rotation
  tftWidth = tft.width();                       // get tft width
  tftHeight = tft.height();                     // get tft height
  xSprite = tftHeight / 100.0 * 20;
  ySprite = tftHeight / 100.0 * 20;
  if (tftWidth >= 320 || tftHeight >= 320)      // big screen (i.e. WT32-SC01 - 480 x 320)
  {textSize = 2;}
  else if (tftWidth < 320 || tftHeight < 320)   // small screen (i.e. LILYGO TTGO-T-Display - 240 x 135)
  {textSize = 1;}
}

// **************************************************************************************
// Screen - init
void screenInit()
{
  tft.fillScreen(0x0000);                       // set desired background color
  tft.setTextFont(1);                           // select font
  tft.setTextSize(textSize);
  tft.setTextDatum(MC_DATUM);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.drawString("5-Way Tactile Switch Test", tftWidth / 2, tftHeight / 100.0 * 5);
  tft.drawLine(0, tftHeight / 100.0 * 10, tftWidth, tftHeight / 100.0 * 10, 0x8410);
  tft.drawLine(0, tftHeight / 100.0 * 90, tftWidth, tftHeight / 100.0 * 90, 0x8410);
  tft.drawString("Push button to highlight icon", tftWidth / 2, tftHeight / 100.0 * 95);
}

// **************************************************************************************
void SpriteTriangleGrayCreate()                 // triangle gray
{
  SpriteTriangleGray.setColorDepth(8);
  SpriteTriangleGray.createSprite(xSprite, ySprite);
  SpriteTriangleGray.fillSprite(0x0000);
  SpriteTriangleGray.fillTriangle(xSprite / 2, xSprite, ySprite, 0, 0, 0, 0x630C);
  SpriteTriangleGray.setPivot(xSprite / 2, ySprite / 2);
}

// **************************************************************************************
void SpriteTriangleRedCreate()                 // triangle red
{
  SpriteTriangleRed.setColorDepth(8);
  SpriteTriangleRed.createSprite(xSprite, ySprite);
  SpriteTriangleRed.fillSprite(0x0000);
  SpriteTriangleRed.fillTriangle(xSprite / 2, xSprite, ySprite, 0, 0, 0, 0x8000);
  SpriteTriangleRed.setPivot(xSprite / 2, ySprite / 2);
}

// **************************************************************************************
void SpriteTriangleGreenCreate()                 // triangle green
{
  SpriteTriangleGreen.setColorDepth(8);
  SpriteTriangleGreen.createSprite(xSprite, ySprite);
  SpriteTriangleGreen.fillSprite(0x0000);
  SpriteTriangleGreen.fillTriangle(xSprite / 2, xSprite, ySprite, 0, 0, 0, 0x0400);
  SpriteTriangleGreen.setPivot(xSprite / 2, ySprite / 2);
}

// **************************************************************************************
void SpriteCircleGrayCreate()                   // circle gray
{
  SpriteCircleGray.setColorDepth(8);
  SpriteCircleGray.createSprite(xSprite, ySprite);
  SpriteCircleGray.fillSprite(0x0000);
  SpriteCircleGray.fillCircle(xSprite / 2, ySprite / 2, xSprite / 2, 0x630C);
  SpriteCircleGray.setPivot(xSprite / 2, ySprite / 2);
}

// **************************************************************************************
void SpriteCircleGreenCreate()                   // circle green
{
  SpriteCircleGreen.setColorDepth(8);
  SpriteCircleGreen.createSprite(xSprite, ySprite);
  SpriteCircleGreen.fillSprite(0x0000);
  SpriteCircleGreen.fillCircle(xSprite / 2, ySprite / 2, xSprite / 2, 0x0400);
  SpriteCircleGreen.setPivot(xSprite / 2, ySprite / 2);
}
